"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const graphql_1 = require("graphql");
class JgqlError extends Error {
    constructor(errorData) {
        super();
        this._graphQLErrors = null;
        this._responseError = null;
        this._errorMessage = null;
        if (errorData.response) {
            if (errorData.response.data &&
                Array.isArray(errorData.response.data.errors)) {
                this._graphQLErrors = errorData.response.data.errors.map((d) => new graphql_1.GraphQLError(d.message));
            }
            else {
                this._responseError = {
                    status: errorData.response.status,
                    data: errorData.response.data,
                };
            }
        }
        else if (typeof errorData === 'string') {
            this._errorMessage = errorData;
        }
        this.message = this.toString();
        this.__proto__ = JgqlError.prototype;
    }
    get type() {
        if (this._graphQLErrors) {
            return 'GRAPHQL';
        }
        else if (this._responseError) {
            return 'RESPONSE';
        }
        else if (this._errorMessage) {
            return 'STRING';
        }
        return 'REQUEST';
    }
    get graphQLErrors() {
        return this._graphQLErrors;
    }
    get responseError() {
        return this._responseError;
    }
    toString() {
        if (this._graphQLErrors) {
            return this._graphQLErrors.map(e => e.message).join('\n');
        }
        else if (this._responseError) {
            return this._responseError.data;
        }
        else if (this._errorMessage) {
            return this._errorMessage;
        }
        else {
            return "Couldn't connect, make sure you are online.";
        }
    }
}
exports.JgqlError = JgqlError;
const augmentRequestDefault = (request) => __awaiter(this, void 0, void 0, function* () { return request; });
const processResponseDefault = (response) => __awaiter(this, void 0, void 0, function* () { return response; });
const processErrorDefault = (error) => __awaiter(this, void 0, void 0, function* () { return error.message; });
exports.createSendRequestFunction = ({ getUrl, augmentRequest = augmentRequestDefault, processResponse = processResponseDefault, processError = processErrorDefault, }) => (query, variables) => __awaiter(this, void 0, void 0, function* () {
    const request = { method: 'POST', url: getUrl(), data: { query, variables } };
    const augmentedRequest = yield augmentRequest(request);
    try {
        const { data: response } = yield axios_1.default(augmentedRequest);
        if (response.errors) {
            throw response.errors;
        }
        const processedResponse = yield processResponse(response);
        return processedResponse.data;
    }
    catch (error) {
        const jgqlError = new JgqlError(error);
        throw processError(jgqlError);
    }
});
