import * as React from 'react';
import { SendRequestFunction, JgqlError } from './sendRequestFunction';
export declare type VariableGetter<V> = () => V | Promise<V>;
export interface InitialState {
    state: 'INITIAL';
}
export interface LoadingState {
    state: 'LOADING';
}
export interface SuccessState<D extends object> {
    state: 'SUCCESS';
    response: D;
}
export interface ErrorState {
    state: 'ERROR';
    error: JgqlError;
}
export declare type JgqlData<D extends object> = InitialState | LoadingState | SuccessState<D> | ErrorState;
export interface JgqlDataProp<D extends object> {
    data?: JgqlData<D>;
}
export interface FetchDataProps<V extends object> {
    refetchData?: (variables?: V) => void;
    registerVariableGetter?: (getter: VariableGetter<V>) => void;
    sendRequest?: SendRequestFunction;
}
export declare type JgqlProps<D extends object, V extends object> = JgqlDataProp<D> & FetchDataProps<V>;
export interface OptionsType<V extends object> {
    dontAutofetch?: boolean;
    getVariables?: VariableGetter<V>;
}
export interface RefetchDataParams<D extends object, V extends object = object> {
    variables?: V;
    processData?: (data: D) => D | Promise<D>;
}
export declare const withJgql: <D extends object = object, V extends object = object>(query?: string | undefined, options?: OptionsType<V> | undefined) => <PP extends JgqlProps<D, V>, P extends object = Pick<PP, Exclude<keyof PP, "data" | "sendRequest" | "refetchData" | "registerVariableGetter">>>(WrappedComponent: React.ComponentType<PP>) => React.StatelessComponent<P>;
