"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const hoistNonReactStatic = require('hoist-non-react-statics');
const withJgqlManual_1 = require("./withJgqlManual");
exports.withJgql = (query, options) => (WrappedComponent) => {
    class JgqlEnhancedFetchingComponent extends React.Component {
        constructor() {
            super(...arguments);
            this.variableGetter = null;
            this.isCompMounted = false;
            this.state = {
                state: 'INITIAL',
            };
            this.registerVariableGetter = (getter) => {
                this.variableGetter = getter;
            };
            this.refetchData = ({ variables, processData, } = {}) => {
                if (!query) {
                    return;
                }
                const newState = {
                    state: 'LOADING',
                };
                if (!this.isCompMounted) {
                    return;
                }
                this.setState(newState, () => __awaiter(this, void 0, void 0, function* () {
                    try {
                        let vars;
                        if (variables) {
                            vars = variables;
                        }
                        else if (this.variableGetter) {
                            vars = yield this.variableGetter();
                        }
                        else if (options && options.getVariables) {
                            vars = yield options.getVariables();
                        }
                        else {
                            vars = {};
                        }
                        const data = yield this.props.sendRequest(query, vars);
                        let processedData;
                        if (processData) {
                            processedData = yield processData(data);
                        }
                        else {
                            processedData = data;
                        }
                        if (!this.isCompMounted) {
                            return;
                        }
                        this.setState({
                            state: 'SUCCESS',
                            response: processedData,
                        });
                    }
                    catch (error) {
                        if (!this.isCompMounted) {
                            return;
                        }
                        this.setState({
                            state: 'ERROR',
                            error,
                        });
                    }
                }));
            };
        }
        componentDidMount() {
            this.isCompMounted = true;
            if (!(options && options.dontAutofetch)) {
                this.refetchData();
            }
        }
        componentWillUnmount() {
            this.isCompMounted = false;
        }
        render() {
            return (React.createElement(WrappedComponent, Object.assign({}, this.props, { data: this.state, refetchData: this.refetchData, registerVariableGetter: this.registerVariableGetter, sendRequest: this.props.sendRequest })));
        }
    }
    const WrappedWrapper = withJgqlManual_1.withJgqlManual(JgqlEnhancedFetchingComponent);
    hoistNonReactStatic(JgqlEnhancedFetchingComponent, WrappedComponent);
    hoistNonReactStatic(WrappedWrapper, JgqlEnhancedFetchingComponent);
    return WrappedWrapper;
};
